#define _CANVAS

#define DISPLAY_LIST_SIZE 20
#define TEXTURE_MAP_LIST_SIZE 10
#define TRUE 1
#define FALSE 0
#define YES 1
#define NO 0
#define glRGB(x, y, z)	glColor3ub((GLubyte)x, (GLubyte)y, (GLubyte)z)
#define dRadToDeg(x) (((float)x)*57.29577951308f)
#define PI 3.14159265359
#define GRID_SIZE 2000
#define NUMBER_OF_BALLS 50
#define OBJECT_LIST_SIZE 10
#define EXPLOSION_SIZE 10

typedef double MATRIX4X4 [4][4];
typedef double MATRIX4X1 [4][1];
typedef double MATRIX1X4 [1][4];

typedef struct Point
{
	GLfloat x,y;
}Point;
typedef struct DISPLAY_LIST_STRUCT
{
	char name[40];
	GLint id;
	GLint available;
}DISPLAY_LIST_TYPE;

typedef struct POLYGON_STRUCT
{
	GLfloat vertexes[4][3];
struct POLYGON_STRUCT *next;
} POLYGON_STRUCT_TYPE;

typedef struct TURTLE_STRUCT
{
	char command[1];
	struct TURTLE_STRUCT *next;
}TURTLE_STRUCT_TYPE;

typedef struct REFLECTION_SPLINE_PATH_STRUCT
{
GLfloat m_t;
GLfloat ray_begin_x;
GLfloat ray_begin_y;
GLfloat ray_begin_z;
GLfloat ray_end_x;
GLfloat ray_end_y;
GLfloat ray_end_z;
GLfloat ray_intersection_x;
GLfloat ray_intersection_y;
GLfloat ray_intersection_z;
GLfloat ray_reflection_x;
GLfloat ray_reflection_y;
GLfloat ray_reflection_z;
}REFLECTION_SPLINE_PATH_STRUCT_TYPE;

typedef struct BEZIER_CONTROL_POINT_STRUCT
{
	GLfloat Px[4];
	GLfloat Py[4];
	GLfloat Pz[4];
} BEZIER_CONTROL_POINT_STRUCT_TYPE;

typedef struct LATHE_CONTROL_POINT_STRUCT
{
GLfloat Px[4];
GLfloat Py[4];
GLfloat Pz[4];
struct LATHE_CONTROL_POINT_STRUCT *next;
} LATHE_CONTROL_POINT_STRUCT_TYPE;

typedef struct LATHE_CATMULL_ROM_CURVE_STRUCT
{
	LATHE_CONTROL_POINT_STRUCT_TYPE *head;
	GLfloat m_t;
} LATHE_CATMULL_ROM_CURVE_STRUCT_TYPE;


typedef struct VECTOR_STRUCT
{
	GLfloat m_x;
	GLfloat m_y;
	GLfloat m_z;
} VECTOR_STRUCT_TYPE;

typedef struct OBJECT_STRUCT{
	GLint m_id;
	GLfloat m_x;
	GLfloat m_y;
	GLfloat m_z;
	GLfloat m_t;
	GLfloat m_Vox;
	GLfloat m_Voy;
	GLfloat m_Vo;
	GLfloat m_Xo;
	GLfloat m_Yo;
	GLint m_yaw;
	GLfloat m_radsFromEast;
	GLint m_pitch;
	GLfloat m_available_flag;
	GLfloat active_flag;
	GLint m_type;
	GLfloat m_cpu_seconds;
} OBJECT_STRUCT_TYPE;

typedef struct POINT_LIST_STRUCT{
	GLfloat x,y;
	GLfloat angle;
	struct POINT_LIST_STRUCT *next;
	struct POINT_LIST_STRUCT *previous;
} POINT_LIST_STRUCT_TYPE;

/*Scene Graph*/

typedef struct treenode
{
	GLfloat m[16];
	char parent_key[30];
	char node_key[30];
	void (*f)();
	void (*engine)();
	struct treenode *sibling;
	struct treenode *child;
	struct treenode *most_recent_child;
}treenode;

/*Camera Coordinates*/
typedef struct Camera
{
GLfloat m_Translate_x;
GLfloat m_Translate_y;
GLfloat	m_Translate_z;

GLfloat m_TranslationUnit;
GLfloat m_radsFromEast;
GLfloat m_camera_direction_y;
GLfloat m_camera_height_y;
}Camera;

/*
typedef struct explosion{
	GLfloat x;
	GLfloat y;
	GLfloat z;
	GLfloat scale_x;
	GLfloat scale_y;
	GLfloat scale_z;
	GLfloat m_cpu_seconds;
	GLint available_flag;
	GLint active_flag;
}EXPLOSION_TYPE;

EXPLOSION_TYPE objExplosion[EXPLOSION_SIZE];
*/

class Canvas
{
public:
void box(GLfloat w,GLfloat h,GLfloat d);
void setLighting();
void setDiffuseMaterialColor(GLfloat red, GLfloat green, GLfloat blue,GLfloat alpha);
void setAmbientMaterialColor(GLfloat red, GLfloat green, GLfloat blue,GLfloat alpha);
void setSpecularMaterialColor(GLfloat red, GLfloat green, GLfloat blue,GLfloat alpha);
void setMaterialShininess(GLfloat shininess);
void brassMaterial();
void goldMaterial();
void bronzeMaterial();
void chromeMaterial();
void copperMaterial();
void silverMaterial();
void pewterMaterial();
void blackPlasticMaterial();
void sphere(GLfloat radius);
void errorHandler(char *module);
void saveBitmap(char *filename);
GLubyte* loadDIBitmap(const char *filename, BITMAPINFO **info);    
GLuint textureLoad(char *filename,GLboolean alpha,  GLenum    minfilter,     GLenum    magfilter,     GLenum    wrap);      
void setTextureMapping();
void setBackgroundColor(GLfloat red, GLfloat green, GLfloat blue);
void setColor(GLfloat red, GLfloat green, GLfloat blue);
void setBlending();
void forward(GLfloat dist, int isVisible);
void setTurn(GLint angle);
void turn(GLint angle);
void moveTo(GLfloat x, GLfloat y);
void lineRel(float dx, float dy);
void lineTo(GLfloat x, GLfloat y);
void nGON(int n, GLfloat cx, GLfloat cy, GLfloat radius, GLfloat rotAngle);
void drawArch(Point center, float radius, float startAngle, float sweep);
void reduceToUnit(GLfloat vector[3]);
void calcNormal(VECTOR_STRUCT_TYPE *vertex1, VECTOR_STRUCT_TYPE *vertex2,VECTOR_STRUCT_TYPE *vertex3,GLfloat out[3]);
void calcNormal2(GLfloat v[3][3], GLfloat out[3]);
void cone(GLfloat radius,GLfloat height);
void cylinder(GLfloat radius,GLfloat height);

VECTOR_STRUCT_TYPE* parameterizeSplineCurve(LATHE_CONTROL_POINT_STRUCT_TYPE *ctrl, GLfloat t);
double Matrix4X1_1X4(MATRIX4X1 *matrix1, MATRIX1X4 *matrix2);
void Matrix4X1_4X4(MATRIX4X1 *result, MATRIX4X1 *matrix1, MATRIX4X4 *matrix2);
void Matrix1X4_4X4(MATRIX1X4 *result, MATRIX1X4 *matrix1, MATRIX4X4 *matrix2);
VECTOR_STRUCT_TYPE* rotate(GLfloat angle,GLfloat ux,GLfloat uy,GLfloat uz, VECTOR_STRUCT_TYPE *vector);
GLint reserveDisplayListId(char *name);
GLint findDisplayListId(char *name);
GLint reserveTextureMap(char *name,char *filename);
GLint findTextureMap(char *name);
void sinc(char *Name);
void textureCylinder(GLfloat radius,GLfloat height);
void textureSphere(GLfloat radius);
void toroidal_Spiral(char *Name);
void buildGrid(char *Name,int size);
void ellipsoid(char *Name);
void torus(char *Name);
void helix(char *Name);
void polyhedra(char *Name);
void trefoilKnot(char *Name);
void traverseSceneGraph(treenode *root);
treenode * node(char *node_key);
void addChild(treenode *parentNode, treenode *childNode);
void cleanSceneGraph(treenode *root);
int getAvailableBall();
void launchBall(GLint Yaw,GLint Pitch,GLfloat Vo);
VECTOR_STRUCT_TYPE* getRandomPosition();
void buildObjectList(char *Name);
void drawObjectList();
void empty();
void buildMissile();
//int getAvailableExplosion();
//void drawExplosion();
protected:

	GLint miWidth;
	GLint miHeight;
	GLint m_CurrDegree;
	TURTLE_STRUCT_TYPE *turtle_head;
	POLYGON_STRUCT_TYPE *head;
	REFLECTION_SPLINE_PATH_STRUCT_TYPE spline;
	LATHE_CATMULL_ROM_CURVE_STRUCT_TYPE lathe_spline;
	Camera myCamera;

	DISPLAY_LIST_TYPE display_list[DISPLAY_LIST_SIZE];
	DISPLAY_LIST_TYPE texture_map_list[TEXTURE_MAP_LIST_SIZE];

	OBJECT_STRUCT_TYPE objList[10];
	
	OBJECT_STRUCT_TYPE objBall[NUMBER_OF_BALLS];

	treenode *root;

	Point CP;

};
