#ifdef WIN32
#include "windows.h"
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include "canvas.h"
#include "lathe.h"

void initializeLatheCurve(){
/*
use http://www.relia.net/~davepamn/splinecalculator.html to define
catmoll rom spline control points.

replace CONTROL_POINT_STRUCT_TYPE with LATHE_CONTROL_POINT_STRUCT


  Note: The first point and the last point need to be 0,0,0

*/

LATHE_CONTROL_POINT_STRUCT_TYPE *current;

cvs.lathe_spline.m_t=0.0f;
cvs.lathe_spline.head = (LATHE_CONTROL_POINT_STRUCT_TYPE *) malloc(sizeof(LATHE_CONTROL_POINT_STRUCT_TYPE));

current = cvs.lathe_spline.head;

current->next=NULL;
current->Px[0]=0;
current->Px[1]=-15;
current->Px[2]=-10;
current->Px[3]=0;

current->Py[0]=0;
current->Py[1]=4;
current->Py[2]=3;
current->Py[3]=10;

current->Pz[0]=0;
current->Pz[1]=0;
current->Pz[2]=0;
current->Pz[3]=0;

current->next=(LATHE_CONTROL_POINT_STRUCT_TYPE *) malloc(sizeof(LATHE_CONTROL_POINT_STRUCT_TYPE));
current = current->next;
current->next=NULL;
current->Px[0]=-15;
current->Px[1]=-10;
current->Px[2]=0;
current->Px[3]=5;

current->Py[0]=4;
current->Py[1]=3;
current->Py[2]=10;
current->Py[3]=2;

current->Pz[0]=0;
current->Pz[1]=0;
current->Pz[2]=0;
current->Pz[3]=0;

current->next=(LATHE_CONTROL_POINT_STRUCT_TYPE *) malloc(sizeof(LATHE_CONTROL_POINT_STRUCT_TYPE));
current = current->next;
current->next=NULL;
current->Px[0]=-10;
current->Px[1]=0;
current->Px[2]=5;
current->Px[3]=10;

current->Py[0]=3;
current->Py[1]=10;
current->Py[2]=2;
current->Py[3]=1;

current->Pz[0]=0;
current->Pz[1]=0;
current->Pz[2]=0;
current->Pz[3]=0;

current->next=(LATHE_CONTROL_POINT_STRUCT_TYPE *) malloc(sizeof(LATHE_CONTROL_POINT_STRUCT_TYPE));
current = current->next;
current->next=NULL;
current->Px[0]=0;
current->Px[1]=5;
current->Px[2]=10;
current->Px[3]=15;

current->Py[0]=10;
current->Py[1]=2;
current->Py[2]=1;
current->Py[3]=5;

current->Pz[0]=0;
current->Pz[1]=0;
current->Pz[2]=0;
current->Pz[3]=0;

current->next=(LATHE_CONTROL_POINT_STRUCT_TYPE *) malloc(sizeof(LATHE_CONTROL_POINT_STRUCT_TYPE));
current = current->next;
current->next=NULL;
current->Px[0]=5;
current->Px[1]=10;
current->Px[2]=15;
current->Px[3]=0;

current->Py[0]=2;
current->Py[1]=1;
current->Py[2]=5;
current->Py[3]=0;

current->Pz[0]=0;
current->Pz[1]=0;
current->Pz[2]=0;
current->Pz[3]=0;


current->next=NULL;

}

void myLatheObject(char *Name)
{
	GLfloat angle;
	LATHE_CONTROL_POINT_STRUCT_TYPE *current;
	VECTOR_STRUCT_TYPE *vector1;
	VECTOR_STRUCT_TYPE *vector2;
	VECTOR_STRUCT_TYPE *vector3;
	VECTOR_STRUCT_TYPE *vector4;

	float vertexes[4][3];
	float normal[4];

	GLfloat t;
	GLint lathe_id;

	lathe_id=reserveDisplayListId("Lathe");

	if(lathe_id==-1)
	{
		printf("\nNo more Display List ids available\n");
		return;
	}

	glNewList(lathe_id,GL_COMPILE);

	setAmbientMaterialColor(0.0f,0.0f,0.5f,1.0f);
	setDiffuseMaterialColor(0.0f,0.0f,0.8f,1.0f);
	
	//cvs.Box(3,3,3);

	current=cvs.lathe_spline.head;
	while(current!=NULL)
	{
		//for (t=0.0f;t<=1.1f;t+=0.1)
		for(t=0.0f;t<1.0;t=t+0.1)
		{

			//clockwise winding
			//t=0.5f;
			vector1=parameterizeSplineCurve(current,t);
			vector4=parameterizeSplineCurve(current,t+0.1);

			glPushMatrix();
				glTranslatef(vector1->m_x,vector1->m_y,vector1->m_z);
				glutSolidSphere(1.0f,10.0f,10.0f);
			glPopMatrix();

			vector2=rotate(10,1,0,0,vector1);
			vector3=rotate(10,1,0,0,vector4);

			vertexes[0][0]=vector1->m_x;
			vertexes[0][1]=vector1->m_y;
			vertexes[0][2]=vector1->m_z;

			vertexes[1][0]=vector2->m_x;
			vertexes[1][1]=vector2->m_y;
			vertexes[1][2]=vector2->m_z;

			vertexes[2][0]=vector3->m_x;
			vertexes[2][1]=vector3->m_y;
			vertexes[2][2]=vector3->m_z;

			vertexes[3][0]=vector4->m_x;
			vertexes[3][1]=vector4->m_y;
			vertexes[3][2]=vector4->m_z;
			calcNormal2(vertexes,normal);

			for(angle=0; angle<=360; angle+=10)
			{
				glPushMatrix();

				glRotatef(angle,1,0,0);

				glBegin(GL_POLYGON);
					glNormal3f(normal[0],normal[1],normal[2]);
					glVertex3f(vector1->m_x,vector1->m_y,vector1->m_z);
					glVertex3f(vector2->m_x,vector2->m_y,vector2->m_z);
					glVertex3f(vector3->m_x,vector3->m_y,vector3->m_z);
					glVertex3f(vector4->m_x,vector4->m_y,vector4->m_z);
				glEnd();

				glPopMatrix();
			}
			

			free(vector1);
			free(vector2);
			free(vector3);
			free(vector4);
			
		}

		current=current->next;
	}
	glEndList();

}
void processLathe()
{
	static int spin;
	spin=(spin+5)%360;

	glTranslatef(0,0,-20);
	glRotatef(spin,0,1,0);
	glCallList(findDisplayListId("Lathe"));	
}
