#ifdef WIN32
#include "windows.h"
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "terrain.h"

// find y value
float getTerrainHeight(float x, float z, float *pa, float *ra)
{
	int r, c;
	float y;

	for (r = 0; r < TERRAIN_COUNT - 1; r++)
	{
		for (c = 0; c < TERRAIN_COUNT - 1; c++)
		{
			// square
			if (x >= LookUp[r][c].xMin && x <= LookUp[r][c].xMax && z >= LookUp[r][c].zMin && z <= LookUp[r][c].zMax)
			{
				//printf("r = %d, c = %d\n", r, c);
				// triange
				if (LookUp[r][c].zC - x - z >= 0)
				{
					y = (LookUp[r][c].k1 -  x * LookUp[r][c].n1[0] - z * LookUp[r][c].n1[2]) / LookUp[r][c].n1[1];
					*pa = -LookUp[r][c].pitch1;
					*ra = -LookUp[r][c].roll1;
					//printf("k1\n");
				}
				else
				{
					y = (LookUp[r][c].k2 -  x * LookUp[r][c].n2[0] - z * LookUp[r][c].n2[2]) / LookUp[r][c].n2[1];
					//printf("k2\n");
					*pa = -LookUp[r][c].pitch2;
					*ra = -LookUp[r][c].roll2;
				}
				return y;
			}
		}
	}
	return 0.0f;
}
void normalTerrain(int r, int c)
{
	// upper left
	if (r > 0 && c > 0)
	{
		Terrain[r][c].n[0] += LookUp[r-1][c-1].n2[0];
		Terrain[r][c].n[1] += LookUp[r-1][c-1].n2[1];
		Terrain[r][c].n[2] += LookUp[r-1][c-1].n2[2];
	}

	// upper right
	if (r > 0 && c < TERRAIN_COUNT - 1)
	{
		Terrain[r][c].n[0] += LookUp[r-1][c].n1[0];
		Terrain[r][c].n[1] += LookUp[r-1][c].n1[1];
		Terrain[r][c].n[2] += LookUp[r-1][c].n1[2];

		Terrain[r][c].n[0] += LookUp[r-1][c].n2[0];
		Terrain[r][c].n[1] += LookUp[r-1][c].n2[1];
		Terrain[r][c].n[2] += LookUp[r-1][c].n2[2];
	}

	// lower left
	if (r < TERRAIN_COUNT - 1 && c > 0)
	{
		Terrain[r][c].n[0] += LookUp[r][c-1].n1[0];
		Terrain[r][c].n[1] += LookUp[r][r-1].n1[1];
		Terrain[r][c].n[2] += LookUp[r][c-1].n1[2];	

		Terrain[r][c].n[0] += LookUp[r][c-1].n2[0];
		Terrain[r][c].n[1] += LookUp[r][c-1].n2[1];
		Terrain[r][c].n[2] += LookUp[r][c-1].n2[2];	
	}

	// lower right
	if (r < TERRAIN_COUNT - 1 && c < TERRAIN_COUNT - 1)
	{
		Terrain[r][c].n[0] += LookUp[r][c].n1[0];
		Terrain[r][c].n[1] += LookUp[r][c].n1[1];
		Terrain[r][c].n[2] += LookUp[r][c].n1[2];
	}

	reduceToUnit(Terrain[r][c].n);
}

// color terrain
void colorTerrain(int r , int c)
{
	if (Terrain[r][c].p[1] < 5.0f)
	{
		Terrain[r][c].c[0] = 0.784f/2;
		Terrain[r][c].c[1] = 0.675f/2;
		Terrain[r][c].c[2] = 0.533f/2;
	}
	else if (Terrain[r][c].p[1] < 10.0f)
	{
		Terrain[r][c].c[0] = 0.824f/2;
		Terrain[r][c].c[1] = 0.714f/2;
		Terrain[r][c].c[2] = 0.573f/2;
	}
	else if (Terrain[r][c].p[1] < 15.0f)
	{
		Terrain[r][c].c[0] = 0.902f/2;
		Terrain[r][c].c[1] = 0.792f/2;
		Terrain[r][c].c[2] = 0.651f/2;
	}
	else
	{
		Terrain[r][c].c[0] = 0.941f/2;
		Terrain[r][c].c[1] = 0.831f/2;
		Terrain[r][c].c[2] = 0.690f/2;
	}
}
// calculate normal for 3 points
void calcTerrainNormal(float p1[3], float p2[3], float p3[3], float n[3])
{
	float v1[3], v2[3];

	v1[0] = p2[0] - p1[0];
	v1[1] = p2[1] - p1[1];
	v1[2] = p2[2] - p1[2];

	v2[0] = p1[0] - p3[0];
	v2[1] = p1[1] - p3[1];
	v2[2] = p1[2] - p3[2];

	n[0] = v1[1]*v2[2] - v1[2]*v2[1];
	n[1] = v1[2]*v2[0] - v1[0]*v2[2];
	n[2] = v1[0]*v2[1] - v1[1]*v2[0];

	reduceToUnit(n);
	//printf("normal = %f, %f, %f\n", n[0], n[1], n[2]);
}

void drawTerrain()
{
	TP *tp;
	int r;
	int c;

	//glBindTexture(GL_TEXTURE_2D, TextureOne);
	//glEnable(GL_TEXTURE_2D);

	tp = Terrain[0];
	for (r = 0; r < (TERRAIN_COUNT - 1); r++)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for (c = 0; c < TERRAIN_COUNT; c++)
		{
			//glColor3fv(tp[0].c);
			glNormal3fv(tp[0].n);
			glTexCoord2f(TERRAIN_STRIP * r / (TERRAIN_COUNT - 1), TERRAIN_STRIP * c / (TERRAIN_COUNT - 1));
			
			glVertex3fv(tp[0].p);

			//glColor3fv(tp[TERRAIN_COUNT].c);
			setDiffuseMaterialColor(tp[TERRAIN_COUNT].c[0],tp[TERRAIN_COUNT].c[1],tp[TERRAIN_COUNT].c[2],1);
			setAmbientMaterialColor(0.2,0.2,0.2,1);
			setSpecularMaterialColor(0.3,0.3,0.3,1);
			setMaterialShininess(10);
			glNormal3fv(tp[TERRAIN_COUNT].n);
			
			glTexCoord2f(TERRAIN_STRIP * (r + 1) / (TERRAIN_COUNT - 1), TERRAIN_STRIP * c / (TERRAIN_COUNT - 1));
			glVertex3fv(tp[TERRAIN_COUNT].p);

			tp++;
		}
		glEnd();
	}


	
	//glDisable(GL_TEXTURE_2D);
	//glEnable(GL_LIGHTING);
}

// create terrain
void createTerrain()
{
	int r, c;
	GLfloat half = (TERRAIN_COUNT - 1) / -2;
	GLfloat x = half;
	GLfloat z = half;

	// intialize terrain
	for (r = 0; r < TERRAIN_COUNT; r++)
	{
		for (c = 0; c < TERRAIN_COUNT; c++)
		{
			// points
			Terrain[r][c].p[0] = x * TERRAIN_SPACE; // x
			Terrain[r][c].p[1] = 0.0f; // y
			Terrain[r][c].p[2] = z * TERRAIN_SPACE; // z
			x++;
			// normals
			Terrain[r][c].n[0] = 0.0f; // x
			Terrain[r][c].n[1] = 1.0f; // y
			Terrain[r][c].n[2] = 0.0f; // z
			// color
			Terrain[r][c].c[0] = 0.863f; // red
			Terrain[r][c].c[1] = 0.753f; // green
			Terrain[r][c].c[2] = 0.612f; // blue
		}
		x = half;
		z++;
	}

	// fractal divide terrain
	divideTerrain(0, TERRAIN_COUNT - 1, 0, TERRAIN_COUNT - 1, TERRAIN_RANGE, TERRAIN_ROUGHNESS);

	// create look up information
	for (r = 0; r < TERRAIN_COUNT - 1; r++)
	{
		for (c = 0; c < TERRAIN_COUNT - 1; c++)
		{
			createLookUp(r, c);
		}
	}

	// calculate normals and colors for terrain
	for (r = 0; r < TERRAIN_COUNT; r++)
	{
		for (c = 0; c < TERRAIN_COUNT; c++)
		{
			normalTerrain(r, c);
			colorTerrain(r, c);
		}
	}
}

float calculateAngle(float n[3], float v[3])
{
	float a;

	a = (n[0] * v[0] + n[1] * v[1] + n[2] * v[2]);
	a = acos(a);
	a = a * 180.0f / 3.14f;
	return a;
}


// divide terrain
void divideTerrain(int r1, int r2, int c1, int c2, int range, float H)
{
	int rMid, cMid; // mid points
	float average; // average of points
	float r;

	rMid = (r1 + r2) / 2;
	cMid = (c1 + c2) / 2;

	// sqaure step
	average = Terrain[r1][c1].p[1] + Terrain[r1][c2].p[1] + Terrain[r2][c1].p[1] + Terrain[r2][c2].p[1];
	average = average / 4.0;
	//printf("average = %f\n", average);
	r = (rand() % range) / 10.0f;
	//if (rand() % 2)
	//{
	//	r = -r;
	//}
	//printf("random = %f\n", r);
	Terrain[rMid][cMid].p[1] = average + r;
	printf("height = %f\n", Terrain[rMid][cMid].p[1]);

	range = range * (2 * exp(-H));
	//printf("new range %i\n", range);

	if (r2 - r1 > 2)
	{
		divideTerrain(r1, rMid, c1, cMid, range, H);
		divideTerrain(rMid, r2, c1, cMid, range, H);
		divideTerrain(r1, rMid, cMid, c2, range, H);
		divideTerrain(rMid, r2, cMid, c2, range, H);
	}		
}

// create look up information
void createLookUp(int r , int c)
{
	float zv[3] = {0.0f, 0.0f, -1.0f};
	float xv[3] = {1.0f, 0.0f, 0.0f};
	
	// sqare
	LookUp[r][c].xMax = Terrain[r][c+1].p[0];
	LookUp[r][c].xMin = Terrain[r][c].p[0];
	LookUp[r][c].zMax = Terrain[r+1][c].p[2];
	LookUp[r][c].zMin = Terrain[r][c].p[2];
	LookUp[r][c].zC = Terrain[r][c+1].p[2] + Terrain[r][c+1].p[0];

	// triangle 1
	calcTerrainNormal(Terrain[r][c+1].p, Terrain[r+1][c].p, Terrain[r][c].p, LookUp[r][c].n1);
	LookUp[r][c].k1 = (Terrain[r][c+1].p[0] * LookUp[r][c].n1[0]) + (Terrain[r][c+1].p[1] * LookUp[r][c].n1[1]) + (Terrain[r][c+1].p[2] * LookUp[r][c].n1[2]);
	//printf("k1 = %f\n", LookUp[r][c].k1);
	//LookUp[r][c].k1 = (Terrain[r+1][c].p[0] * LookUp[r][c].n1[0]) + (Terrain[r+1][c].p[1] * LookUp[r][c].n1[1]) + (Terrain[r+1][c].p[2] * LookUp[r][c].n1[2]);
	//printf("k1 = %f\n", LookUp[r][c].k1);
	//LookUp[r][c].k1 = (Terrain[r][c].p[0] * LookUp[r][c].n1[0]) + (Terrain[r][c].p[1] * LookUp[r][c].n1[1]) + (Terrain[r][c].p[2] * LookUp[r][c].n1[2]);
	//printf("k1 = %f\n", LookUp[r][c].k1);
	// pitch
	LookUp[r][c].pitch1 = 90.0f - calculateAngle(LookUp[r][c].n1, zv);
	LookUp[r][c].roll1 = 90.0f - calculateAngle(LookUp[r][c].n1, xv);

	// triangle 2
	calcTerrainNormal(Terrain[r][c+1].p, Terrain[r+1][c+1].p, Terrain[r+1][c].p, LookUp[r][c].n2);
	LookUp[r][c].k2 = (Terrain[r][c+1].p[0] * LookUp[r][c].n2[0]) + (Terrain[r][c+1].p[1] * LookUp[r][c].n2[1]) + (Terrain[r][c+1].p[2] * LookUp[r][c].n2[2]);
	//printf("k2 = %f\n", LookUp[r][c].k2);
	//LookUp[r][c].k1 = (Terrain[r+1][c+1].p[0] * LookUp[r][c].n2[0]) + (Terrain[r+1][c+1].p[1] * LookUp[r][c].n2[1]) + (Terrain[r+1][c+1].p[2] * LookUp[r][c].n2[2]);
	//printf("k2 = %f\n", LookUp[r][c].k2);
	//LookUp[r][c].k2 = (Terrain[r+1][c].p[0] * LookUp[r][c].n2[0]) + (Terrain[r+1][c].p[1] * LookUp[r][c].n2[1]) + (Terrain[r+1][c].p[2] * LookUp[r][c].n2[2]);
	//printf("k2 = %f\n", LookUp[r][c].k2);
	LookUp[r][c].pitch2 = 90.0f - calculateAngle(LookUp[r][c].n2, zv);
	LookUp[r][c].roll2 = 90.0f - calculateAngle(LookUp[r][c].n2, xv);
}
